@@ -6,8 +6,8 @@ import json
 import re
 from collections import ChainMap
 
+from ruamel.yaml.comments import CommentedMap
 from ruamel.yaml.main import round_trip_dump as yaml_dump
-from ruamel.yaml.main import round_trip_load as yaml_load
 
 attributes_to_ignore = ["resource_id", "name", "tags", "arn"]
 
@@ -17,8 +17,9 @@ def resolve_argument_bindings(
     sls_file_data,
     complete_tf_resource_map,
     complete_resource_map,
-    complete_list_of_variables,
+    complete_dict_of_variables,
     file,
+    dumper,
 ):
     idem_resource_type_resource_id_separator = (
         hub.tf_idem_auto.utils.idem_resource_type_resource_id_separator
@@ -42,7 +43,7 @@ def resolve_argument_bindings(
                         complete_resource_map,
                         idem_resource_type,
                         resource_attribute_value,
-                        complete_list_of_variables,
+                        complete_dict_of_variables,
                     )
                 if resource_attribute_key == "resource_id":
                     sls_resource_data = complete_resource_map.get(
@@ -51,7 +52,7 @@ def resolve_argument_bindings(
                     if sls_resource_data:
                         resource_attribute[
                             resource_attribute_key
-                        ] = f"{{{{ params.get(\"{sls_resource_data.get('resource_path')}\", '')}}}}"
+                        ] = f"{{{{ params.get(\"{sls_resource_data.get('resource_path')}\")}}}}"
 
                 if tf_resource:
                     (
@@ -63,12 +64,14 @@ def resolve_argument_bindings(
                     # tf_resource_value = list(list(tf_resource.values())[0].values())[
                     #     0
                     # ].get(resource_attribute_key)
-                    # TODO: Have to  remove the below code: line 58-62
-                    if (
-                        isinstance(tf_resource_value, str)
-                        and "jsonencode" in tf_resource_value
+                    if isinstance(tf_resource_value, str) and re.search(
+                        r"\${jsonencode\(var\.[\w-]+\)}", tf_resource_value
                     ):
-                        continue
+                        tf_resource_value = re.sub(
+                            r"\${jsonencode\(var\.[\w-]+\)}",
+                            lambda jsonencode_string: f'"${{var.{jsonencode_string.group()[17:-2]}}}"',
+                            tf_resource_value,
+                        )
                     if tf_resource_value:
                         parameterized_value = check_if_eligible_for_parameter_binding(
                             hub,
@@ -96,9 +99,7 @@ def resolve_argument_bindings(
             list_of_comments.extend(
                 look_for_possible_improvements(hub, tf_resource, resource_attributes)
             )
-            sls_file_data_with_comment = yaml_load(
-                yaml_dump({item: sls_file_data[item]})
-            )
+            sls_file_data_with_comment = CommentedMap({item: sls_file_data[item]})
             if len(list_of_comments) > 0:
                 sls_file_data_with_comment.yaml_set_start_comment(
                     "\n" + "\n".join(list_of_comments)
@@ -106,12 +107,7 @@ def resolve_argument_bindings(
             else:
                 sls_file_data_with_comment.yaml_set_start_comment("\n")
 
-            yaml_dump(
-                sls_file_data_with_comment,
-                file,
-                indent=hub.tf_idem_auto.utils.INDENTATION,
-                block_seq_indent=hub.tf_idem_auto.utils.INDENTATION,
-            )
+            yaml_dump(sls_file_data_with_comment, file, Dumper=dumper)
 
 
 def look_for_possible_improvements(hub, tf_resource, resource_attributes):
@@ -161,7 +157,7 @@ def check_if_eligible_for_argument_binding(
     complete_resource_map,
     complete_tf_resource_map,
     attribute_value,
-    variables_data=None,
+    complete_dict_of_variables=None,
 ):
     if isinstance(attribute_value, dict) or isinstance(attribute_value, list):
         if isinstance(attribute_value, dict):
@@ -171,7 +167,7 @@ def check_if_eligible_for_argument_binding(
                     complete_resource_map,
                     complete_tf_resource_map,
                     attribute_value[item],
-                    variables_data,
+                    complete_dict_of_variables,
                 )
             return attribute_value
         else:
@@ -182,7 +178,7 @@ def check_if_eligible_for_argument_binding(
                     complete_resource_map,
                     complete_tf_resource_map,
                     item,
-                    variables_data,
+                    complete_dict_of_variables,
                 )
                 new_attributes.append(arg_binded_item)
             return new_attributes
@@ -195,7 +191,7 @@ def check_if_eligible_for_argument_binding(
                     complete_resource_map,
                     complete_tf_resource_map,
                     attribute_value[item],
-                    variables_data,
+                    complete_dict_of_variables,
                 )
             return json.dumps(attribute_value)
         return check_argument_binding(
@@ -203,7 +199,7 @@ def check_if_eligible_for_argument_binding(
             complete_resource_map,
             complete_tf_resource_map,
             attribute_value,
-            variables_data,
+            complete_dict_of_variables,
         )
 
 
@@ -258,7 +254,7 @@ def check_if_eligible_for_parameter_binding(
             tf_resource_val = json.loads(tf_resource_val)
             attribute_val_itr = json.loads(attribute_val_itr)
             for item in tf_resource_val:
-                tf_resource_val[item] = check_if_eligible_for_parameter_binding(
+                parameterized_value = check_if_eligible_for_parameter_binding(
                     hub,
                     tf_resource_val[item],
                     attribute_key,
@@ -267,6 +263,8 @@ def check_if_eligible_for_parameter_binding(
                     complete_resource_map,
                     complete_tf_resource_map,
                 )
+                if parameterized_value:
+                    tf_resource_val[item] = parameterized_value
                 if attribute_key == "tags" and isinstance(attribute_value, list):
                     tf_resource_val = convert_tags_dict_list(tf_resource_val)
             return json.dumps(tf_resource_val)
@@ -277,15 +275,19 @@ def check_if_eligible_for_parameter_binding(
         return parameterize(tf_resource_val, attribute_key, attribute_value)
 
 
-def is_attr_value_in_variables(idem_resource_val, variables_data):
-    for attr_key, attribute_value in variables_data.items():
-        if attribute_value == idem_resource_val:
+def is_attr_value_in_variables(idem_resource_val, complete_dict_of_variables):
+    for attr_key, attribute_value in complete_dict_of_variables.items():
+        if isinstance(attribute_value, str) and attribute_value == idem_resource_val:
             return attr_key
     return None
 
 
 def check_argument_binding(
-    hub, complete_resource_map, idem_resource_type, attribute_value, variables_data=None
+    hub,
+    complete_resource_map,
+    idem_resource_type,
+    attribute_value,
+    complete_dict_of_variables=None,
 ):
     idem_resource_type_resource_id_separator = (
         hub.tf_idem_auto.utils.idem_resource_type_resource_id_separator
@@ -302,9 +304,9 @@ def check_argument_binding(
                 f"{tf_auto_supported_resource_type}{idem_resource_type_resource_id_separator}{attribute_value}"
             ]
             return f"${{{next(iter(referred_resource_dict['resource'])).replace('.present', '')}:{referred_resource_dict['resource_path']}:{referred_resource_dict['type']}}}"
-        elif variables_data:
+        elif complete_dict_of_variables:
             attribute_in_variable = is_attr_value_in_variables(
-                attribute_value, variables_data
+                attribute_value, complete_dict_of_variables
             )
             if attribute_in_variable:
                 return f'{{{{params.get("{attribute_in_variable}")}}}}'
@@ -318,12 +320,39 @@ def convert_var_to_param(var_input):
 
 def convert_local_to_param(var_input):
     variable_name = var_input.group()[2:-1]
-    return f'{{{{ params.get("{variable_name.replace(".", "_")}") }}}}'
+    variable_name = variable_name.replace(".", "_")
+    return f'{{{{ params.get("{variable_name}") }}}}'
 
 
 def convert_local_to_param_dict(var_input):
     variable_name = var_input.group()[2:-1]
-    return f'{{{{ params.get("{variable_name.replace(".", "_") + "_dict"}") }}}}'
+    variable_name = variable_name.replace(".", "_") + "_dict"
+    return f'{{{{ params.get("{variable_name}") }}}}'
+
+
+def handle_count_index_in_tags(additional_tags, attribute_value):
+    attribute_value_dict = convert_tags_list_dict(attribute_value)
+    additional_tags_updated = {}
+    for tag_key in additional_tags:
+        tag_value = additional_tags.get(tag_key)
+        if "-${count.index}" in tag_key:
+            tag_key = replace_count_index_in_tags(
+                tag_key, attribute_value_dict.get(tag_key)
+            )
+        if "-${count.index}" in additional_tags.get(tag_key):
+            tag_value = replace_count_index_in_tags(
+                tag_value, attribute_value_dict.get(tag_key)
+            )
+        additional_tags_updated[tag_key] = tag_value
+    return additional_tags_updated
+
+
+def replace_count_index_in_tags(tag_key, original_tag_value):
+    split_list = original_tag_value.split("-")
+    for count_index in split_list:
+        if count_index.isnumeric():
+            tag_key = tag_key.replace("-${count.index}", f"-{count_index}")
+    return tag_key
 
 
 def parameterize(resolved_value, attribute_key, attribute_value):
@@ -337,20 +366,34 @@ def parameterize(resolved_value, attribute_key, attribute_value):
         if re.search(r"\${merge\(", resolved_value):
             resolved_value = resolved_value.replace("'", '"')
             try:
-                additional_tags = json.loads(
+                format_additional_tags_str = (
                     resolved_value.replace("${merge(local.tags,,", "")
                     .replace(",)}", "")
-                    .replace('"clusterName"', "'clusterName'")
-                    .replace("{{", "+")
-                    .replace("}}", "+")
+                    .replace("{{ ", "")
+                    .replace("}} ", "")
+                    .replace(" {{", "")
+                    .replace(" }}", "")
+                    .replace("{{", "")
+                    .replace("}}", "")
+                    .replace("'", '"')
                 )
+
+                format_additional_tags_str_p = format_additional_tags_str.replace(
+                    '("', '(\\"'
+                ).replace('")', '\\")')
+                additional_tags = json.loads(format_additional_tags_str_p)
+                if "-${count.index}" in format_additional_tags_str_p:
+                    additional_tags = handle_count_index_in_tags(
+                        additional_tags, attribute_value
+                    )
+
                 parameterized_value = (
-                    f'{{{{params.get("local_tags") + {convert_tags_dict_list(additional_tags)}}}}}'
+                    f'{{{{ params.get("local_tags") + {adjust_format_of_additional_tags(json.dumps(convert_tags_dict_list(additional_tags)))}}}}}'
                     if isinstance(attribute_value, list)
-                    else f'{{{{params.get("local_tags_dict") + {dict(additional_tags)}}}}}'
+                    else f'{{{{ params.get("local_tags_dict") + {adjust_format_of_additional_tags(json.dumps(dict(additional_tags)))}}}}}'
                 )
-            except:
-                print("I an im")
+            except Exception as e:
+                print("Exception in loading additional tags :: ", e)
         elif "local." in resolved_value:
             json_loads = json.loads(json.dumps(resolved_value))
             if isinstance(attribute_value, list):
@@ -365,6 +408,13 @@ def parameterize(resolved_value, attribute_key, attribute_value):
     return parameterized_value
 
 
+def convert_tags_list_dict(tags_list):
+    tags_dict = {}
+    for tag in tags_list:
+        tags_dict[tag.get("Key")] = tag.get("Value")
+    return tags_dict
+
+
 def convert_tags_dict_list(tags_dict):
     tags_list = []
     for tag in tags_dict:
@@ -377,6 +427,18 @@ def convert_tags_dict_list(tags_dict):
     return tags_list
 
 
+def adjust_format_of_additional_tags(tag_str):
+    # Handle if params is  at beginning or ending or middle
+    tag_str = tag_str.replace('params.get(\\"', '"+params.get("')
+    tag_str = tag_str.replace('"params.get(\\"', 'params.get("')
+    tag_str = tag_str.replace('"+params.get(\\"', 'params.get("')
+    tag_str = tag_str.replace('\\")', '")+"')
+    tag_str = tag_str.replace('+""', "")
+    tag_str = tag_str.replace('""+', "")
+
+    return tag_str
+
+
 def fix_format_of_additional_tags(input_tag):
     if not any(substring in input_tag for substring in ["{{", "}}"]):
         return input_tag
