@@ -5,6 +5,7 @@ from collections import ChainMap
 from collections import OrderedDict
 
 import yaml
+from ruamel.yaml import RoundTripDumper
 
 
 def convert_tf_file_to_sls(
@@ -156,8 +157,9 @@ def convert_tf_file_to_sls(
     if "locals" in obj:
         local_variables = obj.get("locals")
         parameterized_locals = parameterize_values(local_variables, {})
-        for key, value in parameterized_locals.items():
-            variables.update({f"local_{key}": value})
+        for local in parameterized_locals:
+            for key, value in local.items():
+                variables.update({f"local_{key}": value})
     os.makedirs(os.path.dirname(output_sls_file_path), exist_ok=True)
     if "variables.tf" in input_tf_file_path:
         output_sls_file_path = (
@@ -182,8 +184,10 @@ def parameterize_values(obj, new_dict):
             new_dict[k] = parameterize_values(v, {})
             # return new_dict
     elif isinstance(obj, (list, set, tuple)):
+        new_list = []
         for item in obj:
-            parameterize_values(item, new_dict)
+            new_list.append(parameterize_values(item, new_dict))
+        return new_list
     else:
         return obj
     return new_dict
@@ -233,8 +237,8 @@ def convert_tf_files_to_sls(
     dir_list = os.walk(root)
     complete_resource_map = {}
     complete_tf_resource_map = {}
-    complete_list_of_variables = {}
-    name_of_files_in_module = []
+    complete_dict_of_variables = {}
+    name_of_files_in_module = set()
     for path, subdirs, files in dir_list:
         for file in files:
             file_path = os.path.join(path, file)
@@ -261,8 +265,8 @@ def convert_tf_files_to_sls(
                 if idem_resource_id_tf_resource_map is not None:
                     complete_tf_resource_map.update(idem_resource_id_tf_resource_map)
                 if variables is not None:
-                    complete_list_of_variables.update(variables)
-                name_of_files_in_module.append(f"sls.{file[:-3]}")
+                    complete_dict_of_variables.update(variables)
+                name_of_files_in_module.add(f"sls.{file[:-3]}")
 
     # Perform argument binding of all values in each resource of file, wherever possible
     resolve_argument_binding(
@@ -270,7 +274,12 @@ def convert_tf_files_to_sls(
         complete_resource_map,
         complete_tf_resource_map,
         module_output_directory_path,
-        complete_list_of_variables,
+        complete_dict_of_variables,
+    )
+
+    # Change values of bool type into string type
+    complete_dict_of_variables = hub.tf_idem_auto.utils.change_bool_values_to_string(
+        complete_dict_of_variables
     )
 
     # Generate parent init file
@@ -278,9 +287,18 @@ def convert_tf_files_to_sls(
         os.path.dirname(f"{module_output_directory_path}/init.sls"),
         exist_ok=True,
     )
+    files_to_exclude_in_parent_init = {"sls.variables"}
     with open(f"{module_output_directory_path}/init.sls", "w") as _file:
         yaml.dump(
-            {"include": sorted(name_of_files_in_module)},
+            {
+                "include": sorted(
+                    list(
+                        name_of_files_in_module.difference(
+                            files_to_exclude_in_parent_init
+                        )
+                    )
+                )
+            },
             _file,
             default_flow_style=False,
         )
@@ -292,7 +310,7 @@ def convert_tf_files_to_sls(
     )
     with open(f"{module_output_directory_path}/params/variables.sls", "w") as file1:
         yaml.dump(
-            complete_list_of_variables, file1, default_flow_style=False, Dumper=MyDumper
+            complete_dict_of_variables, file1, default_flow_style=False, Dumper=MyDumper
         )
 
     generate_resource_ids_file(hub, module_output_directory_path, complete_resource_map)
@@ -309,7 +327,7 @@ def resolve_argument_binding(
     complete_resource_map,
     complete_tf_resource_map,
     sls_directory,
-    complete_list_of_variables,
+    complete_dict_of_variables,
 ):
     dir_list = os.walk(sls_directory)
     for path, subdir, files in dir_list:
@@ -329,8 +347,9 @@ def resolve_argument_binding(
                         sls_file_data,
                         complete_tf_resource_map,
                         complete_resource_map,
-                        complete_list_of_variables,
+                        complete_dict_of_variables,
                         file,
+                        MyDumper2,
                     )
 
             else:
@@ -369,6 +388,8 @@ def generate_resource_ids_file(
 def generate_params_init_file(module_output_directory_path):
     param_list = []
     for file in os.listdir(f"{module_output_directory_path}/params/"):
+        if file == "init.sls":
+            continue
         param_list.append(file[:-4])
     init_dict = {"include": sorted(param_list)}
     with open(f"{module_output_directory_path}/params/init.sls", "w") as file2:
@@ -436,6 +457,18 @@ class MyDumper1(yaml.SafeDumper):
         super().write_plain(text)
 
 
+class MyDumper2(RoundTripDumper):
+    def write_line_break(self, data=None):
+        super().write_line_break(data)
+
+        if len(self.indents) == 1:
+            super().write_line_break()
+            super().write_line_break()
+
+    def write_single_quoted(self, text, split=True):
+        super().write_plain(text)
+
+
 def convert_var_to_param(var_input):
     variable_names = []
     variable_name = var_input.group()[6:-1]
